import 'package:_private_core/_private_core.dart';

import 'package:_private_core_network/network_resources/resources.dart';
import 'package:dio/dio.dart';

import 'model/chat_message_detault.dart';

class MyAppEndpoint {
  MyAppEndpoint._();
  static String getDefaultMessagesForChat() =>
      "v2/chat/getDefaultMessagesForChat";
  static String setDefaultMessageForChat() =>
      "v2/chat/setDefaultMessageForChat";
  static String createFirstMessage() =>
      "v2/chat/createFirstMessage";
}

abstract class MyAppApi {
  Future<NetworkResponse> getDefaultMessagesForChat(params);
  Future<NetworkResponse> setDefaultMessageForChat(params);
  Future<NetworkResponse> createFirstMessage(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> getDefaultMessagesForChat(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          MyAppEndpoint.getDefaultMessagesForChat(),
          queryParameters: params);
      return NetworkResponse.fromResponse(
        response,
        converter: (json) => ChatMessageDefault.fromJson(json),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setDefaultMessageForChat(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(MyAppEndpoint.setDefaultMessageForChat(), data: params);
      return NetworkResponse.fromResponse(
        response,
        value: response.statusCode == 200,
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createFirstMessage(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(MyAppEndpoint.createFirstMessage(), data: params);
      return NetworkResponse.fromResponse(
        response,
        value: response.statusCode == 200,
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }
}
