import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:dio/dio.dart';

import 'models/models.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String googlePlaceAutoComplete() => "googlePlaceAutoComplete";
  static String googlePlaceSearch() => "googlePlaceSearch";
}

abstract class MyAppApi {
  ///Google place
  Future<NetworkResponse> googlePlaceSeachPlaceByText(String keyword,
      [String? types]);
  Future<NetworkResponse> googlePlaceSeachPlaceDetailById(placeId);
  Future<NetworkResponse> googleGeocode(latlng);

  ///Google place pass our sever
  Future<NetworkResponse> googlePlaceSearch(params);
  Future<NetworkResponse> googlePlaceSearchById(params);
  Future<NetworkResponse> googlePlaceAutoComplete(String keyword,
      {String? countryCode, List<String>? types, String? language});
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> googlePlaceSeachPlaceByText(String keyword,
      [String? types]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/place/textsearch/json',
          queryParameters: {
            'query': keyword,
            'key': googleApiKey,
            'type': types
          });
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) => (json as List<dynamic>)
              .map<SeachPlaceResultModel>(
                  (i) => SeachPlaceResultModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSeachPlaceDetailById(placeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/place/details/json',
          queryParameters: {'place_id': placeId, 'key': googleApiKey});
      return NetworkResponse.fromResponse(response,
          prefix: 'result',
          converter: (json) => SeachPlaceResultDetailModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googleGeocode(latlng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/geocode/json',
          queryParameters: {'latlng': latlng, 'key': googleApiKey});
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) => (json as List<dynamic>)
              .map<SeachPlaceResultDetailModel>(
                  (i) => SeachPlaceResultDetailModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(_MyAppEndpoint.googlePlaceSearch(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) => (json as List<dynamic>)
              .map<SeachPlaceResultModel>(
                  (i) => SeachPlaceResultModel.fromJson(i))
              .toList());
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSearchById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(_MyAppEndpoint.googlePlaceSearch(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          prefix: 'result',
          converter: (json) => SeachPlaceResultDetailModel.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceAutoComplete(String keyword,
      {String? countryCode, List<String>? types, String? language}) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      var queryParameters = {
        'value': keyword,
        'types': types ?? ['address']
      };
      if (countryCode != null) {
        queryParameters.addAll({
          'country': countryCode,
        });
      }
      if (language != null) {
        queryParameters.addAll({
          'language': language,
        });
      }
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(_MyAppEndpoint.googlePlaceAutoComplete(),
              queryParameters: queryParameters);
      if (response.data['status'] == 'OK') {
        return NetworkResponse.fromResponse(response,
            prefix: 'predictions',
            converter: (json) => (json as List<dynamic>)
                .map<AutocompletePrediction>(
                    (i) => AutocompletePrediction.fromJson(i))
                .toList());
      } else {
        return NetworkResponse();
      }
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }
}
