import 'dart:collection';

import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'app_api.dart';
import 'models/models.dart';

class GooglePlaceRepo {
  GooglePlaceRepo._();

  static GooglePlaceRepo? _instance;

  factory GooglePlaceRepo([MyAppApi? api]) {
    _instance ??= GooglePlaceRepo._();
    _instance!._api = api ?? MyAppApiImp();
    return _instance!;
  }

  late MyAppApi _api;

  Map<String, dynamic> _mapParms([Map? value]) {
    Map<String, dynamic> params = {};
    if (value != null && value.isNotEmpty) params.addAll(HashMap.from(value));
    params
      ..removeWhere((key, value) => value == null)
      ..addAll({
        'platformId': valuePlatformByConfig(),
        'languageCode': appPrefs.languageCode,
      });
    return params;
  }

  Future<List<SeachPlaceResultModel>> googlePlaceSeachPlaceByText(
      String keyword,
      [types]) async {
    NetworkResponse response =
        await _api.googlePlaceSeachPlaceByText(keyword, types);
    return response.data ?? [];
  }

  Future<SeachPlaceResultDetailModel?> googlePlaceSeachPlaceDetailById(
      placeId) async {
    NetworkResponse response =
        await _api.googlePlaceSeachPlaceDetailById(placeId);
    return response.data;
  }

  Future<List<SeachPlaceResultDetailModel>> googleGeocode(latlng) async {
    NetworkResponse response = await _api.googleGeocode(latlng);
    return response.data ?? [];
  }

  //Search text
  // {'query': keyword, 'type': types}
  Future<List<SeachPlaceResultModel>> googlePlaceSearch(params) async {
    NetworkResponse response = await _api.googlePlaceSearch(params);
    return response.data ?? [];
  }

  //Get place detail
  // {'placeId': placeId
  Future<SeachPlaceResultDetailModel?> googlePlaceSearchById(params) async {
    NetworkResponse response = await _api.googlePlaceSearchById(params);
    return response.data;
  }

  Future<List<AutocompletePrediction>> googlePlaceAutoComplete(String keyword,
      {String? countryCode, List<String>? types, String? language}) async {
    NetworkResponse response = await _api.googlePlaceAutoComplete(
      keyword,
      countryCode: countryCode,
      types: types,
      language: language,
    );
    return response.data ?? [];
  }
}
