enum SportProgramsStatus { none, paused, completed }

class SportPrograms {
  String? id;
  int? cardType;
  int? numberOfExercices;
  int? totalDuration;
  int? programOwnerProId;
  int? programPropertyId;
  int? isPublished;
  String? programPictureUrl;
  String? name;
  String? description;
  List<ProgramContents>? programContents;
  ProgramProperty? programProperty;

  SportProgramsStatus get status {
    if (programContents!.any((e) => e.pausedTime != null)) {
      return SportProgramsStatus.paused;
    }
    if (!programContents!.any((e) => e.isCompleted == false)) {
      return SportProgramsStatus.completed;
    }
    return SportProgramsStatus.none;
  }

  int get indexPaused {
    if (status == SportProgramsStatus.paused) {
      return programContents!
          .indexOf(programContents!.firstWhere((e) => e.pausedTime != null));
    }
    return 0;
  }

  SportPrograms(
      {this.id,
      this.cardType,
      this.numberOfExercices,
      this.totalDuration,
      this.programOwnerProId,
      this.programPropertyId,
      this.isPublished,
      this.programPictureUrl,
      this.name,
      this.description,
      this.programContents,
      this.programProperty});

  SportPrograms.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) this.id = json["id"];
    if (json["cardType"] is int) this.cardType = json["cardType"];
    if (json["numberOfExercices"] is int) {
      this.numberOfExercices = json["numberOfExercices"];
    }
    if (json["totalDuration"] is int) {
      this.totalDuration = json["totalDuration"];
    }
    if (json["programOwnerProId"] is int) {
      this.programOwnerProId = json["programOwnerProId"];
    }
    if (json["programPropertyId"] is int) {
      this.programPropertyId = json["programPropertyId"];
    }
    if (json["isPublished"] is int) this.isPublished = json["isPublished"];
    if (json["programPictureUrl"] is String) {
      this.programPictureUrl = json["programPictureUrl"];
    }
    if (json["name"] is String) this.name = json["name"];
    if (json["description"] is String) this.description = json["description"];
    if (json["programContents"] is List) {
      this.programContents = json["programContents"] == null
          ? null
          : (json["programContents"] as List)
              .map((e) => ProgramContents.fromJson(e))
              .toList();
    }
    if (json["programProperty"] is Map) {
      this.programProperty = json["programProperty"] == null
          ? null
          : ProgramProperty.fromJson(json["programProperty"]);
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data["id"] = this.id;
    data["cardType"] = this.cardType;
    data["numberOfExercices"] = this.numberOfExercices;
    data["totalDuration"] = this.totalDuration;
    data["programOwnerProId"] = this.programOwnerProId;
    data["programPropertyId"] = this.programPropertyId;
    data["isPublished"] = this.isPublished;
    data["programPictureUrl"] = this.programPictureUrl;
    data["name"] = this.name;
    data["description"] = this.description;
    if (this.programContents != null) {
      data["programContents"] =
          this.programContents?.map((e) => e.toJson()).toList();
    }
    if (this.programProperty != null) {
      data["programProperty"] = this.programProperty?.toJson();
    }
    return data;
  }
}

class ProgramProperty {
  int? id;
  String? slug;
  String? video;
  String? lat;
  String? lng;
  String? content;
  String? address;

  ProgramProperty(
      {this.id,
      this.slug,
      this.video,
      this.lat,
      this.lng,
      this.content,
      this.address});

  ProgramProperty.fromJson(Map<String, dynamic> json) {
    if (json["id"] is int) this.id = json["id"];
    if (json["slug"] is String) this.slug = json["slug"];
    if (json["video"] is String) this.video = json["video"];
    if (json["lat"] is String) this.lat = json["lat"];
    if (json["lng"] is String) this.lng = json["lng"];
    if (json["content"] is String) this.content = json["content"];
    if (json["address"] is String) this.address = json["address"];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data["id"] = this.id;
    data["slug"] = this.slug;
    data["video"] = this.video;
    data["lat"] = this.lat;
    data["lng"] = this.lng;
    data["content"] = this.content;
    data["address"] = this.address;
    return data;
  }
}

class ProgramContents {
  String? id;
  int? subcardOrderNumber;
  String? subcardPictureUrl;
  int? exerciceDuration;
  dynamic breakAfter;
  int? numberOfRepetitions;
  dynamic values;
  int? breakBetweenRep;
  dynamic numberOfSeries;
  dynamic materialPictureUrl;
  dynamic exLottieUrl;
  dynamic exGifUrl;
  dynamic exVideoUrl;
  int? isBreak;
  String? subcardTitle;
  String? description;
  String? subcardVideoUrl;
  dynamic materialDescription;

  int? startedTime;
  int? pausedTime;
  int pausedDuration = 0;
  bool isCompleted = false;

  ProgramContents({
    this.id,
    this.subcardOrderNumber,
    this.subcardPictureUrl,
    this.exerciceDuration,
    this.breakAfter,
    this.numberOfRepetitions,
    this.values,
    this.breakBetweenRep,
    this.numberOfSeries,
    this.materialPictureUrl,
    this.exLottieUrl,
    this.exGifUrl,
    this.exVideoUrl,
    this.isBreak,
    this.subcardTitle,
    this.description,
    this.subcardVideoUrl,
    this.materialDescription,
    this.isCompleted = false,
  });

  ProgramContents.fromJson(Map<String, dynamic> json) {
    if (json["id"] is String) this.id = json["id"];
    if (json["subcardOrderNumber"] is int) {
      this.subcardOrderNumber = json["subcardOrderNumber"];
    }
    if (json["subcardPictureUrl"] is String) {
      this.subcardPictureUrl = json["subcardPictureUrl"];
    }
    if (json["exerciceDuration"] is int) {
      this.exerciceDuration = json["exerciceDuration"];
    }
    this.breakAfter = json["breakAfter"];
    if (json["numberOfRepetitions"] is int) {
      this.numberOfRepetitions = json["numberOfRepetitions"];
    }
    this.values = json["values"];
    if (json["breakBetweenRep"] is int) {
      this.breakBetweenRep = json["breakBetweenRep"];
    }
    this.numberOfSeries = json["numberOfSeries"];
    this.materialPictureUrl = json["materialPictureUrl"];
    this.exLottieUrl = json["exLottieUrl"];
    this.exGifUrl = json["exGifUrl"];
    this.exVideoUrl = json["exVideoUrl"];
    if (json["isBreak"] is int) this.isBreak = json["isBreak"];
    if (json["subcardTitle"] is String) {
      this.subcardTitle = json["subcardTitle"];
    }
    if (json["description"] is String) this.description = json["description"];
    if (json["subcardVideoUrl"] is String) {
      this.subcardVideoUrl = json["subcardVideoUrl"];
    }
    this.materialDescription = json["materialDescription"];
    this.startedTime = json["startedTime"];
    this.pausedTime = json["pausedTime"];
    this.pausedDuration = json["pausedDuration"] ?? 0;
    this.isCompleted = json["isCompleted"] ?? false;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data["id"] = this.id;
    data["subcardOrderNumber"] = this.subcardOrderNumber;
    data["subcardPictureUrl"] = this.subcardPictureUrl;
    data["exerciceDuration"] = this.exerciceDuration;
    data["breakAfter"] = this.breakAfter;
    data["numberOfRepetitions"] = this.numberOfRepetitions;
    data["values"] = this.values;
    data["breakBetweenRep"] = this.breakBetweenRep;
    data["numberOfSeries"] = this.numberOfSeries;
    data["materialPictureUrl"] = this.materialPictureUrl;
    data["exLottieUrl"] = this.exLottieUrl;
    data["exGifUrl"] = this.exGifUrl;
    data["exVideoUrl"] = this.exVideoUrl;
    data["isBreak"] = this.isBreak;
    data["subcardTitle"] = this.subcardTitle;
    data["description"] = this.description;
    data["subcardVideoUrl"] = this.subcardVideoUrl;
    data["materialDescription"] = this.materialDescription;
    data["startedTime"] = this.startedTime;
    data["pausedTime"] = this.pausedTime;
    data["pausedDuration"] = this.pausedDuration;
    data["isCompleted"] = this.isCompleted;
    return data;
  }
}
