import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:dio/dio.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String getBillingInfos() => "v2/payment/getBillingInfos";
  static String updateBillingInfos() => "v2/payment/updateBillingInfos";
  static String deleteBillingInfos() => "v2/payment/deleteBillingInfos";
  static String addBillingInfos() => "v2/payment/addBillingInfos";
  static String setDefaultBillingInfos() => "v2/payment/setDefaultBillingInfos";
}

abstract class MyAppApi {
  Future<NetworkResponse> getBillingInfos(params);
  Future<NetworkResponse> updateBillingInfos(params);
  Future<NetworkResponse> deleteBillingInfos(params);
  Future<NetworkResponse> addBillingInfos(params);
  Future<NetworkResponse> setDefaultBillingInfos(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> addBillingInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(_MyAppEndpoint.addBillingInfos(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => PaymentBillingInfo.fromJson(json));
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateBillingInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(_MyAppEndpoint.updateBillingInfos(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setDefaultBillingInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(_MyAppEndpoint.setDefaultBillingInfos(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteBillingInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnect();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(_MyAppEndpoint.deleteBillingInfos(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioException catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getBillingInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClient(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(_MyAppEndpoint.getBillingInfos(), queryParameters: params);
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json as List).map((e) => PaymentBillingInfo.fromJson(e)).toList(),
      );
    } on DioException catch (e) {
      appDebugPrint('DioException: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }
}
